!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
! The massive rotating straight segment
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 

 SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
!---------- ---- 

   IMPLICIT NONE
   INTEGER, INTENT(IN) :: NDIM, IJAC, ICP(*)
   DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
   DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
   DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,*), DFDP(NDIM,*)

   DOUBLE PRECISION x,y,z,xp,yp,zp,lambda,mu,r1,r2,p,s,d1,d2
   DOUBLE PRECISION r1x,r1y,r1z,r2x,r2y,r2z,px,py,pz,sx,sy,sz
   DOUBLE PRECISION d1x,d1y,d1z,d2x,d2y,d2z
   DOUBLE PRECISION cx,cy,cz,dx,dy,dz,ex,ey,ez
   DOUBLE PRECISION vx,vy,vz,vxp,vyp,vzp

   x  = U(1)
   y  = U(2)
   z  = U(3)
   xp = U(4)
   yp = U(5)
   zp = U(6)

   lambda = PAR(1)
   mu = PAR(2)

   r1 = SQRT((x-1)**2 + y**2 + z**2)
   r2 = SQRT((x+1)**2 + y**2 + z**2)

   p = (r1+r2)*r1*r2
   s = 4.d0/(r1+r2)**2

   d1 = 1.d0/p
   d2 = 1.d0/(p*(1-s))

   F(1)= xp
   F(2)= yp
   F(3)= zp
   F(4)= 2*yp + x * ( 1 - 2*mu*d1 ) + lambda*xp
   F(5)=-2*xp + y * ( 1 - 2*mu*d2 ) + lambda*yp
   F(6)=      -2*mu*z*d2            + lambda*zp

   IF(NDIM==6)RETURN

! Set up the linearized equations

   r1x = (x-1)/r1
   r1y = y/r1
   r1z = z/r1

   r2x = (x+1)/r2
   r2y = y/r2
   r2z = z/r2

   px = (r1x+r2x)*r1*r2 + (r1+r2)*r1x*r2 + (r1+r2)*r1*r2x
   py = (r1y+r2y)*r1*r2 + (r1+r2)*r1y*r2 + (r1+r2)*r1*r2y
   pz = (r1z+r2z)*r1*r2 + (r1+r2)*r1z*r2 + (r1+r2)*r1*r2z

   sx = -8*(r1x+r2x)/(r1+r2)**3
   sy = -8*(r1y+r2y)/(r1+r2)**3
   sz = -8*(r1z+r2z)/(r1+r2)**3

   d1x = -px/p**2
   d1y = -py/p**2
   d1z = -pz/p**2

   d2x = -px/(p**2*(1-s)) + sx/(p*(1-s)**2)
   d2y = -py/(p**2*(1-s)) + sy/(p*(1-s)**2)
   d2z = -pz/(p**2*(1-s)) + sz/(p*(1-s)**2)

   cx = 1 - 2*mu*d1 - 2*mu*x*d1x
   cy =             - 2*mu*x*d1y
   cz =             - 2*mu*x*d1z

   dx =             - 2*mu*y*d2x
   dy = 1 - 2*mu*d2 - 2*mu*y*d2y
   dz =             - 2*mu*y*d2z

   ex =          - 2*mu*z*d2x
   ey =          - 2*mu*z*d2y
   ez = -2*mu*d2 - 2*mu*z*d2z

   vx = U(7)       
   vy = U(8)       
   vz = U(9)
   vxp= U(10)       
   vyp= U(11)      
   vzp= U(12)

   F(7) = vxp
   F(8) = vyp
   F(9) = vzp
   F(10)= cx*vx + cy*vy + cz*vz + 2*vyp
   F(11)= dx*vx + dy*vy + dz*vz - 2*vxp
   F(12)= ex*vx + ey*vy + ez*vz

! Scale
   F = F * PAR(11)
 
 END SUBROUTINE FUNC
!---------------------------------------------------------------------- 

 SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC)
!---------- ----

   IMPLICIT NONE
   INTEGER, INTENT(IN) :: NDIM, ICP(*), NBC, IJAC
   DOUBLE PRECISION, INTENT(IN) :: PAR(*), U0(NDIM), U1(NDIM)
   DOUBLE PRECISION, INTENT(OUT) :: FB(NBC), DBC(NBC,*)

   DOUBLE PRECISION fm

! Periodicity boundary conditions
   FB(1:6) = U0(1:6) - U1(1:6)

! Floquet eigenvalue/vector boundary relation
   fm = PAR(4)
   FB(7:12) = U1(7:12) - fm*U0(7:12)
 
 END SUBROUTINE BCND
!---------------------------------------------------------------------- 

 SUBROUTINE ICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,FI,IJAC,DINT)
!---------- ----

   IMPLICIT NONE
   INTEGER, INTENT(IN) :: NDIM, ICP(*), NINT, IJAC
   DOUBLE PRECISION, INTENT(IN) :: PAR(*)
   DOUBLE PRECISION, INTENT(IN) :: U(NDIM), UOLD(NDIM), UDOT(NDIM), UPOLD(NDIM)
   DOUBLE PRECISION, INTENT(OUT) :: FI(NINT), DINT(NINT,*)

! Integral phase condition
   FI(1) = DOT_PRODUCT(U(1:6),UPOLD(1:6))

! Integral Floquet eigenfunction normalization
   FI(2) = -PAR(10)+DOT_PRODUCT(U(7:12),U(7:12))

 END SUBROUTINE ICND
!---------------------------------------------------------------------- 

 SUBROUTINE PVLS(NDIM,U,PAR)
!---------- ----

   IMPLICIT NONE
   INTEGER, INTENT(IN) :: NDIM
   DOUBLE PRECISION, INTENT(IN) :: U(NDIM)
   DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)

   DOUBLE PRECISION GETP,mu,x,y,z,xp,yp,zp,d1,d2,EU,E,realfm,imagfm,ncmplx
   DOUBLE PRECISION routh,pi
   INTEGER i,j

   mu = PAR(2)

! Get the maximum of the y-coordinate
   PAR(8)=GETP('MAX',2,U)

! Get the maximum of the y-velocity
   PAR(9)=ABS(GETP('MAX',5,U))

   IF(NDIM==12)RETURN

! Maximum real Floquet multiplier: PAR(4)
   PAR(4) = 0
   ncmplx = 0
   DO i=1,NDIM
      imagfm = GETP('EIG',I*2,U)
      IF (imagfm == 0) THEN
         realfm = GETP('EIG',I*2-1,U)
         IF (ABS(realfm) > ABS(PAR(4))) THEN
            PAR(4) = realfm
         ENDIF
      ELSE
         ncmplx = ncmplx + 1
      ENDIF
   ENDDO
   IF (ncmplx == 0) THEN
      ! no complex multipliers mean 6 real multipliers: 1,1, two with
      ! absolute value greater than 1 and two less than one
      PAR(4) = 0
   ELSEIF (ncmplx == 4) THEN
      ! all non-trivial multipliers complex: must be 1 without rounding
      ! errors
      PAR(4) = 1
   ENDIF

!  Put purely imaginary multipliers in PAR(5), PAR(6) and PAR(7)

   j=1
   PAR(5) = 0	
   PAR(6) = 0
   PAR(7) = 0
   pi = 4*ATAN(1d0)
   DO i=1,NDIM
      imagfm = GETP('EIG',I*2,U)
      IF (imagfm > 1d-5) THEN
         realfm = GETP('EIG',I*2-1,U)
         routh = 0.5d0*(1d0-sqrt(69d0)/9d0)
         ! above Routh's ratio we have one period for L4/L5, otherwise 3.
         IF (ABS(realfm) < EPSILON(1d0) .OR. mu < routh .OR. ABS(y) < 0.1) THEN
            PAR(4+j) = 2*pi/imagfm
            j=j+1
         ENDIF
      ENDIF
   ENDDO

 END SUBROUTINE PVLS
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
 SUBROUTINE STPNT
 END SUBROUTINE STPNT

 SUBROUTINE FOPT 
 END SUBROUTINE FOPT
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
