!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
! MRSS:    The massive rotating straight segment
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 

 SUBROUTINE GETRHS(NDIM,U,PAR,F)
!---------- ------

   IMPLICIT NONE
   INTEGER, INTENT(IN) :: NDIM
   DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
   DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)

   DOUBLE PRECISION x,y,xp,yp,mu,r1,r2,p,s

! Define the RHS:
 
   x  = U(1)
   y  = U(2)
   xp = U(3)
   yp = U(4)

   mu = PAR(2)

   r1 = SQRT((x-1)**2 + y**2)
   r2 = SQRT((x+1)**2 + y**2)

   p = (r1+r2)*r1*r2
   s = 4.d0/(r1+r2)**2

   F(1)= xp
   F(2)= yp
   F(3)= 2*yp + x * ( 1 - 2*mu/p )
   F(4)=-2*xp + y * (1 - 2*mu/(p*(1-s)) )

 END SUBROUTINE GETRHS
!----------------------------------------------------------------------

 SUBROUTINE GETJAC(NDIM,U,PAR,DFDU)
!---------- ------

   IMPLICIT NONE
   INTEGER, INTENT(IN) :: NDIM
   DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
   DOUBLE PRECISION, INTENT(OUT) :: DFDU(NDIM,*)

   DOUBLE PRECISION x,y,xp,yp,mu,r1,r2,p,s,d1,d2
   DOUBLE PRECISION r1x,r1y,r2x,r2y,px,py,sx,sy
   DOUBLE PRECISION d1x,d1y,d2x,d2y
   DOUBLE PRECISION cx,cy,dx,dy
   DOUBLE PRECISION vx,vy,vxp,vyp

! Define the Jacobian:

   x  = U(1)
   y  = U(2)
   xp = U(3)
   yp = U(4)

   mu = PAR(2)

   r1 = SQRT((x-1)**2 + y**2)
   r2 = SQRT((x+1)**2 + y**2)

   p = (r1+r2)*r1*r2
   s = 4.d0/(r1+r2)**2

   d1 = 1.d0/p
   d2 = 1.d0/(p*(1-s))

   r1x = (x-1)/r1
   r1y = y/r1

   r2x = (x+1)/r2
   r2y = y/r2

   px = (r1x+r2x)*r1*r2 + (r1+r2)*r1x*r2 + (r1+r2)*r1*r2x
   py = (r1y+r2y)*r1*r2 + (r1+r2)*r1y*r2 + (r1+r2)*r1*r2y

   sx = -8*(r1x+r2x)/(r1+r2)**3
   sy = -8*(r1y+r2y)/(r1+r2)**3

   d1x = -px/p**2
   d1y = -py/p**2

   d2x = -px/(p**2*(1-s)) + sx/(p*(1-s)**2)
   d2y = -py/(p**2*(1-s)) + sy/(p*(1-s)**2)

   cx = 1 - 2*mu*d1 - 2*mu*x*d1x
   cy =             - 2*mu*x*d1y

   dx =             - 2*mu*y*d2x
   dy = 1 - 2*mu*d2 - 2*mu*y*d2y

     DFDU(1,1) = 0.d0
     DFDU(1,2) = 0.d0
     DFDU(1,3) = 1.d0
     DFDU(1,4) = 0.d0

     DFDU(2,1) = 0.d0
     DFDU(2,2) = 0.d0
     DFDU(2,3) = 0.d0
     DFDU(2,4) = 1.d0

     DFDU(3,1) = cx
     DFDU(3,2) = cy
     DFDU(3,3) = 0.d0
     DFDU(3,4) = 2.d0

     DFDU(4,1) = dx
     DFDU(4,2) = dy
     DFDU(4,3) =-2.d0
     DFDU(4,4) = 0.d0

 END SUBROUTINE GETJAC
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
