!---------------------------------------------------------------------- 
!----------------------------------------------------------------------
 INCLUDE "equations.f90"	
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 

 SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
!---------- ---- 
 
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM, IJAC
  INTEGER, INTENT(IN) :: ICP(*)
  DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
  DOUBLE PRECISION, INTENT(IN) :: DFDU(NDIM,*), DFDP(NDIM,*)
  DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)

    DOUBLE PRECISION T
    INTEGER i

! Get the RHS of the dynamical systems
    CALL GETRHS(NDIM,U,PAR,F)

! Scale
    T = PAR(14)

    DO i=1,NDIM
      F(i) = T * F(i)
    ENDDO
 
 END SUBROUTINE FUNC

!---------------------------------------------------------------------- 

 SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC) 
!---------- ---- 
 
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM, NBC, IJAC
  INTEGER, INTENT(IN) :: ICP(*)
  DOUBLE PRECISION, INTENT(IN) :: U0(NDIM), U1(NDIM), PAR(*)
  DOUBLE PRECISION, INTENT(IN) :: DBC(NBC,*)
  DOUBLE PRECISION, INTENT(OUT) :: FB(NDIM)

  INTEGER ND2, ND4, i
  PARAMETER (ND2=2, ND4=4)
  DOUBLE PRECISION ULB(ND4), VLB(ND4), FLB(ND4), eps

! The coordinates of the libration point are in PAR(21)-PAR(22)
    DO i=1,ND2
      ULB(i) = PAR(20+i)
    ENDDO

! Set the velocity components to zero
    DO i=ND2+1,ND4
      ULB(i) = 0.d0
    ENDDO

! The equations for the libration point
    CALL GETRHS(ND4,ULB,PAR,FLB)
  
    DO i=1,ND2
      FB(i) = FLB(ND2+i)
    ENDDO

! The unstable eigenvector of the libration point
    CALL EIGLB(ND4,ULB,VLB,PAR)

! The coordinates of the starting point of the orbit in the unstable manifold
    eps = PAR(13)
    DO i=1,ND4
      FB(ND2+i) = U0(i) - ( ULB(i) + eps*VLB(i) )
    ENDDO

! The coordinates of the endpoint of the orbit in the unstable manifold
    DO i=1,ND4
      FB(ND4+ND2+i) = U1(i) - PAR(30+i)
    ENDDO

 END SUBROUTINE BCND

!---------------------------------------------------------------------- 

 SUBROUTINE ICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,FI,IJAC,DINT)
!---------- ----

  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM, NINT, IJAC
  INTEGER, INTENT(IN) :: ICP(*)
  DOUBLE PRECISION, INTENT(IN) :: U(NDIM), UOLD(NDIM), UDOT(NDIM), UPOLD(NDIM)
  DOUBLE PRECISION, INTENT(IN) :: DINT(NINT,*), PAR(*)
  DOUBLE PRECISION, INTENT(OUT) :: FI(NDIM)

  INTEGER ND4
  PARAMETER (ND4=4)
  DOUBLE PRECISION F(ND4), T

! The length of the orbit in the manifold is put in PAR(12):

    T = PAR(14)
    FI(1) = T*DSQRT( U(3)**2 + U(4)**2 ) - PAR(12)

 END SUBROUTINE ICND

!---------------------------------------------------------------------- 

SUBROUTINE STPNT(NDIM,U,PAR,time)
!--------- -----

  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM
  DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM), PAR(*)
  DOUBLE PRECISION, INTENT(IN) :: time

  INTEGER ND2, ND4, i
  PARAMETER (ND2=2, ND4=4)
  DOUBLE PRECISION mu, T, L, eps
  DOUBLE PRECISION ULB(ND4), VLB(ND4)

    mu = 0.1
    T  = 1e-3
    L  = 1e-3
    eps= 1e-2

    PAR(2) = mu
    PAR(14) = T
    PAR(12) = L
    PAR(13) = eps

! Initial libration point
     ULB(1) = 1.04668d0
     ULB(2) = 0.d0
    DO i=1,ND2
      ULB(ND2+i) = 0.d0
      PAR(20+i) = ULB(i)
    ENDDO

! The unstable eigenvector of the libration point
    CALL EIGLB(NDIM,ULB,VLB,PAR)

! Initial orbit (taken constant, i.e. time-independent
!                since T is very small)
    DO i=1,NDIM
      U(i) = ULB(i) + eps*VLB(i)
    ENDDO

! Initial end point of orbit (taken equal to the start point)
    DO i=1,NDIM
      PAR(30+i) = U(i)
    ENDDO

 END SUBROUTINE STPNT
!---------------------------------------------------------------------- 

 SUBROUTINE PVLS(NDIM,V,PAR)
!---------- ----

  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM
  DOUBLE PRECISION, INTENT(IN) :: V(NDIM)
  DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)

  DOUBLE PRECISION GETP,mu,x,y,z,xp,yp,zp,d1,d2,EU,E

 END SUBROUTINE PVLS
!---------------------------------------------------------------------- 

 SUBROUTINE EIGLB(NDIM,ULB,VLB,PAR)
!---------- -----
 
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM
  DOUBLE PRECISION, INTENT(IN) :: ULB(NDIM)
  DOUBLE PRECISION, INTENT(OUT) :: VLB(NDIM), PAR(*)

  INTEGER ND4, first, index, i, IER
  DOUBLE PRECISION tolerance
  PARAMETER (ND4=4, tolerance=1.d-6)
  INTEGER IV1(ND4)
  COMMON /FIRST1/ first
     
  DOUBLE PRECISION JAC(ND4,ND4), WR(ND4), WI(ND4), Z(ND4,ND4), FV1(ND4)
  DOUBLE PRECISION rev, iev, ss

! Get the Jacobian at the libration point
    CALL GETJAC(ND4,ULB,PAR,JAC)

! Use the AUTO library routine RG to compute the eigenvalues
    CALL RG(ND4,ND4,JAC,WR,WI,1,Z,IV1,FV1,IER)

    index=0
    DO i=1,ND4
      rev = WR(i)
      iev = WI(i)
      IF(rev.GT.tolerance .AND. DABS(iev).LT.tolerance)THEN
        index=i
        GOTO 1
      ENDIF
    ENDDO

 1  IF(index.NE.0)THEN
      DO i=1,ND4
        VLB(i) = Z(i,index)
      ENDDO
!     Put the eigenvalue in PAR(4):
      PAR(4) = WR(index)
    ELSE
       WRITE(6,100)
       WRITE(9,100)
 100   FORMAT(" ERROR: No real, positive eigenvalue")
       STOP
    ENDIF

! Normalize:
    ss=0.d0
    DO i=1,ND4
      ss = ss + VLB(i)**2
    ENDDO
    ss = DSQRT(ss)
    DO i=1,ND4
      VLB(i) = VLB(i)/ss
    ENDDO

! Set the orientation so the x-component is positive
    IF(VLB(1).LT.0.)THEN
      DO i=1,ND4
        VLB(i) = -VLB(i)
      ENDDO
    ENDIF

! Write the eigenvalues/vectors upon first call:
    IF(first.NE.1234)THEN
      WRITE(9,101)
 101  FORMAT(/," Eigenvalues at the libration point:")
      DO i=1,ND4
        WRITE(9,102)i,WR(i),WI(i)
 102    FORMAT(4X,"Eigenvalue",I2,1P2E19.10)
      ENDDO
      WRITE(9,104)
 104  FORMAT(/," Eigenvector of the real positive eigenvalue:")
      WRITE(9,105)(VLB(i),i=1,ND4)
 105  FORMAT(4X,1P3E19.10)
      first=1234
    ENDIF

 END SUBROUTINE EIGLB
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
 SUBROUTINE FOPT 
 END SUBROUTINE FOPT
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 

